/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * TestWindow.java
 *
 * Created on 29-gen-2011, 17.25.42
 */

package electro.view;

import electro.file.CSVFileFilter;
import electro.file.CSVFileReader;
import electro.entities.Coalition;
import electro.core.Coalizer;
import electro.entities.PowerUser;
import electro.exceptions.TimeIntervalsNumberMismatchException;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.io.File;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Random;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.UIManager;

/**
 * MainWindow class
 *
 * @author Andrea Zambon
 *
 * This class implements the model and most of the view of the application (MVC
 * model) The rest of the view is in the ColorLegend and ColorLegendItem classes.
 */
public class MainWindow extends javax.swing.JFrame {

    private static final int graphWidth = 300;
    private static final int graphHeight = 100;

    private JPanel mainPanel;
    private Color graphBackground;

    private ArrayList<PowerUser> users;
    private String filename;
    private int usersNumber;
    private int coalitionNumber;
    private float coalitionSize;
    private boolean running;
    private boolean changed;
    private Coalizer c;
    ArrayList<Coalition> finalSolution;

    /**
     * Constructor for the MainWindow class
     *
     * Creates a MainWindow object. It initializes all internal fields and
     * displays he window on screen
     */
    public MainWindow() {

        //Initialize the file name, the number of users, the number of coalitions
        //the maximum size of each coalition. Also set the flags representing
        //the situation where the algorithm is not running and something was
        //changed since the last run (to force a reload of the users data)
        filename = "";
        usersNumber = 10;
        coalitionNumber = 1;
        coalitionSize = 1.0f;
        running = false;
        changed = true;
        finalSolution = null;

        //Create an empty array as a placeholder for the users data
        users = new ArrayList<PowerUser>();

        //Try to set the user interface to look like the system default interface
        try {

            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        
        catch (Exception ex) {

            //Do nothing. Use the default java interface style
        }

        //Initialize all automatically generated components (do not edit this line)
        initComponents();

        //Create a panel for the scoll portion of the window
        mainPanel = new JPanel();

        //Set the panel as the scrolled element
        scrollPanel.setViewportView(mainPanel);

        //Stack all elements in the main (scrolled) panel
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

        //Create the color for the background of the graphs
        graphBackground = Color.getHSBColor(1.0f, 0f, 0.5f);

        //Initially no solution in the window
        showNoSolution();

        //Show the main window
        setVisible(true);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        scrollPanel = new javax.swing.JScrollPane();
        progressBarPanel = new javax.swing.JPanel();
        progressBar = new javax.swing.JProgressBar();
        stopButton = new javax.swing.JButton();
        startButton = new javax.swing.JButton();
        dataInputPanel = new javax.swing.JPanel();
        loadCSVButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        usersNumberText = new javax.swing.JFormattedTextField();
        coalitionsInputPanel = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        coalitionNumberText = new javax.swing.JFormattedTextField();
        coalitionValueText = new javax.swing.JFormattedTextField();
        legendPanel = new javax.swing.JPanel();
        legendButton = new javax.swing.JButton();
        jLabel4 = new javax.swing.JLabel();
        menuBar = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        menuOpen = new javax.swing.JMenuItem();
        menuLegend = new javax.swing.JMenuItem();
        menuStartStop = new javax.swing.JMenuItem();
        menuQuit = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Electro");
        setExtendedState(1);
        setMinimumSize(new java.awt.Dimension(720, 500));
        setName("Electro"); // NOI18N

        scrollPanel.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollPanel.setToolTipText("A graphical representation of the best solution found so far by the algorithm");
        scrollPanel.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        scrollPanel.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        scrollPanel.setDoubleBuffered(true);
        scrollPanel.setHorizontalScrollBar(null);
        scrollPanel.setMaximumSize(new java.awt.Dimension(300, 32767));
        scrollPanel.setMinimumSize(new java.awt.Dimension(300, 22));
        scrollPanel.setName("Solution"); // NOI18N
        scrollPanel.setPreferredSize(new java.awt.Dimension(300, 32767));

        progressBarPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Computation progress"));
        progressBarPanel.setName("test"); // NOI18N

        progressBar.setToolTipText("The computation progress");
        progressBar.setDoubleBuffered(true);
        progressBar.setEnabled(false);

        javax.swing.GroupLayout progressBarPanelLayout = new javax.swing.GroupLayout(progressBarPanel);
        progressBarPanel.setLayout(progressBarPanelLayout);
        progressBarPanelLayout.setHorizontalGroup(
            progressBarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(progressBarPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(progressBar, javax.swing.GroupLayout.DEFAULT_SIZE, 328, Short.MAX_VALUE)
                .addContainerGap())
        );
        progressBarPanelLayout.setVerticalGroup(
            progressBarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(progressBarPanelLayout.createSequentialGroup()
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        stopButton.setText("Stop");
        stopButton.setToolTipText("Stop the currently running coalization algorithm");
        stopButton.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        stopButton.setEnabled(false);
        stopButton.setMaximumSize(new java.awt.Dimension(44, 32767));
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed(evt);
            }
        });

        startButton.setText("Start");
        startButton.setToolTipText("Start the coalization algorithm to determine the optimal coalization");
        startButton.setEnabled(false);
        startButton.setMaximumSize(new java.awt.Dimension(46, 32767));
        startButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startButtonActionPerformed(evt);
            }
        });

        dataInputPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Power user data input"));
        dataInputPanel.setDoubleBuffered(false);
        dataInputPanel.setName("dataInput"); // NOI18N

        loadCSVButton.setText("Select CSV file");
        loadCSVButton.setToolTipText("Choose the CSV file to load user data from");
        loadCSVButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadCSVButtonActionPerformed(evt);
            }
        });

        jLabel1.setText("Maximum number of users");

        usersNumberText.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0"))));
        usersNumberText.setText("10");
        usersNumberText.setToolTipText("The maximum number of users to pass to the algorithm");
        usersNumberText.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                usersNumberTextPropertyChange(evt);
            }
        });

        javax.swing.GroupLayout dataInputPanelLayout = new javax.swing.GroupLayout(dataInputPanel);
        dataInputPanel.setLayout(dataInputPanelLayout);
        dataInputPanelLayout.setHorizontalGroup(
            dataInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(dataInputPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(loadCSVButton, javax.swing.GroupLayout.PREFERRED_SIZE, 123, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(dataInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel1)
                    .addComponent(usersNumberText, javax.swing.GroupLayout.DEFAULT_SIZE, 193, Short.MAX_VALUE))
                .addContainerGap())
        );
        dataInputPanelLayout.setVerticalGroup(
            dataInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(dataInputPanelLayout.createSequentialGroup()
                .addGroup(dataInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(loadCSVButton, javax.swing.GroupLayout.DEFAULT_SIZE, 52, Short.MAX_VALUE)
                    .addGroup(dataInputPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 18, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(usersNumberText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );

        coalitionsInputPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Coalitions settings"));

        jLabel2.setText("Max number of coalitions");

        jLabel3.setText("Max power consumption");

        coalitionNumberText.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0"))));
        coalitionNumberText.setText("1");
        coalitionNumberText.setToolTipText("The maximum number of coalitions that the algorithm may output");
        coalitionNumberText.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                coalitionNumberTextPropertyChange(evt);
            }
        });

        coalitionValueText.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0.00"))));
        coalitionValueText.setText("1.0");
        coalitionValueText.setToolTipText("The maximum cumulative power consumption of each coalition");
        coalitionValueText.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                coalitionValueTextPropertyChange(evt);
            }
        });

        javax.swing.GroupLayout coalitionsInputPanelLayout = new javax.swing.GroupLayout(coalitionsInputPanel);
        coalitionsInputPanel.setLayout(coalitionsInputPanelLayout);
        coalitionsInputPanelLayout.setHorizontalGroup(
            coalitionsInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(coalitionsInputPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(coalitionsInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel3)
                    .addComponent(jLabel2))
                .addGap(18, 18, 18)
                .addGroup(coalitionsInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(coalitionNumberText, javax.swing.GroupLayout.DEFAULT_SIZE, 133, Short.MAX_VALUE)
                    .addComponent(coalitionValueText, javax.swing.GroupLayout.DEFAULT_SIZE, 133, Short.MAX_VALUE))
                .addContainerGap())
        );
        coalitionsInputPanelLayout.setVerticalGroup(
            coalitionsInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(coalitionsInputPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(coalitionsInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(coalitionNumberText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(coalitionsInputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(coalitionValueText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        legendPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Users colors"));

        legendButton.setText("Show and modify color legend");
        legendButton.setToolTipText("Show the color legend and allow one to change the color of each user");
        legendButton.setEnabled(false);
        legendButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                legendButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout legendPanelLayout = new javax.swing.GroupLayout(legendPanel);
        legendPanel.setLayout(legendPanelLayout);
        legendPanelLayout.setHorizontalGroup(
            legendPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(legendPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(legendButton, javax.swing.GroupLayout.DEFAULT_SIZE, 328, Short.MAX_VALUE)
                .addContainerGap())
        );
        legendPanelLayout.setVerticalGroup(
            legendPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(legendPanelLayout.createSequentialGroup()
                .addComponent(legendButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jLabel4.setText("Best solution found so far");

        jMenu1.setText("Edit");

        menuOpen.setText("Set input file");
        menuOpen.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuOpenActionPerformed(evt);
            }
        });
        jMenu1.add(menuOpen);

        menuLegend.setText("Legend");
        menuLegend.setEnabled(false);
        menuLegend.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuLegendActionPerformed(evt);
            }
        });
        jMenu1.add(menuLegend);

        menuStartStop.setText("Start");
        menuStartStop.setEnabled(false);
        menuStartStop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuStartStopActionPerformed(evt);
            }
        });
        jMenu1.add(menuStartStop);

        menuQuit.setText("Quit");
        menuQuit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuQuitActionPerformed(evt);
            }
        });
        jMenu1.add(menuQuit);

        menuBar.add(jMenu1);

        setJMenuBar(menuBar);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(legendPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(coalitionsInputPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(dataInputPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(stopButton, javax.swing.GroupLayout.DEFAULT_SIZE, 175, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(startButton, javax.swing.GroupLayout.DEFAULT_SIZE, 177, Short.MAX_VALUE))
                    .addComponent(progressBarPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(scrollPanel, javax.swing.GroupLayout.PREFERRED_SIZE, 320, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel4))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                        .addComponent(jLabel4)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(scrollPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 405, Short.MAX_VALUE))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(dataInputPanel, javax.swing.GroupLayout.PREFERRED_SIZE, 89, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(coalitionsInputPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(legendPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(progressBarPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 85, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(stopButton, javax.swing.GroupLayout.DEFAULT_SIZE, 49, Short.MAX_VALUE)
                            .addComponent(startButton, javax.swing.GroupLayout.DEFAULT_SIZE, 49, Short.MAX_VALUE))))
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {startButton, stopButton});

        dataInputPanel.getAccessibleContext().setAccessibleName("dataInput");

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Stop button callback
     *
     * @param evt The event that happened. The callback actually doesw not use
     * this parameter, so any value will be regarded as a left click event.
     *
     * This method handles a click on the stop button. It stops the algorithm
     * (if it is running)
     */
    private void stopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed

        //Make sure that no problems can happen due to concurrent execution
        synchronized (this) {

            //Only stop the execution of the algorithm if it is running
            if (running) {

                //Stop the algorithm
                c.stop();
            }
        }
    }//GEN-LAST:event_stopButtonActionPerformed

    /**
     * Start button callback method
     *
     * @param evt The event that ahhened to the start button. This event is not
     * actually read, so any event will be regarded as a left click event.
     *
     * This method handles a click on the start button. It starts the algorithm
     * with the previously set parameters.
     */
    private void startButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startButtonActionPerformed

        //Try loading the users. Proceed only if this operation is succesful
        if(loadUsers(false)) {

            /*System.out.println("--------- New execution ---------");
            System.out.println("LDF with one user in each coalition: " + coalitionNumber);

            //Try putting all users in the coalitions in a random way.

            //Create an array of coalitions
            Coalition[] coal = new Coalition[coalitionNumber];

            //Create each coalition
            for (int i = 0; i < coalitionNumber; ++i) {

                coal[i] = new Coalition(coalitionSize,
                                        users.get(0).timeIntervalNumber(),
                                        users.size());
            }

            //Get a random number generator
            Random r = new Random();

            boolean found = false;
            boolean repeat = false;

            //Repeat the random assignment attempt for up to 1000 times
            for (int j = 0; j < 1000; ++j) {

                repeat = false;

                //Clear each coalition
                for (int i = 0; i < coalitionNumber; ++i) {

                    coal[i].clear();
                }

                //Assign each user to a random coalition
                for (int i = 0; i < users.size(); ++i) {

                    try {

                        coal[r.nextInt(coalitionNumber)].push(users.get(i));
                    }

                    catch (TimeIntervalsNumberMismatchException ex) {

                        //Should not happen
                    }
                }

                //Check coalition validity
                for (int i = 0; i < coalitionNumber; ++i) {

                    if (!coal[i].isAValidCoalition()) {

                        //Error: invalid assignment
                        repeat = true;
                    }
                }

                //Go back to the beginning of the loop
                if (repeat) {

                    continue;
                }

                float LDF = 0;

                //Compute the LDF of the found assignment
                for (int i = 0; i < coalitionNumber; ++i) {

                    LDF += coal[i].LDF();
                }

                //Write the resulting LDF to standard output
                System.out.println("Random LDF: " + LDF);

                //Remember that a solution was found
                found = true;

                break;
            }

            if (!found) {

                //Write a special message if no solution could be found
                System.out.println("Could not find a valid random assignment");
            }*/
            

            try {

                //Delete any previous solution found
                finalSolution = null;

                //Ready to go
                c = new Coalizer(users, coalitionSize, coalitionNumber, this);

                //Remember that the algorithm is running
                running = true;

                //Update the status of all the items on the window
                updateWindowStatus();

                //Initially the progress bar is in an ideterminate state
                progressBar.setIndeterminate(true);

                //Initially show no solution
                showNoSolution();

                //Go
                c.start();
            }

            catch (TimeIntervalsNumberMismatchException ex) {

                //Show an error message
                JOptionPane.showMessageDialog(this,
                    "Error while initializing the algorithm. A pair of users was" +
                    " found with a different number of time samples. Please make " +
                    "sure that the supplied user power consumption data all have " +
                    "the same number of samples.");
            }
        }
    }//GEN-LAST:event_startButtonActionPerformed

    /**
     * Legend button callback method
     *
     * @param evt The event that happened to the button. This event is not
     * actually read, so any event is regarded as a left click
     *
     * Handles the legend button in the main window
     */
    private void legendButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_legendButtonActionPerformed

        //Load the users if they were not loaded already
        if (loadUsers(false)) {

            //Create a ColorLegend window with a reference to the users array
            //and let it run
            ColorLegend cl = new ColorLegend(users);
        }
    }//GEN-LAST:event_legendButtonActionPerformed

    /**
     * Callback method for the load CSV button
     *
     * @param evt The event that happened to the load CSV button. This event is
     * not actually read by the method, so any event is treated as a left click.
     *
     * Handles a click on the load CSV button. Displays a file chooser to
     * specify a file path.
     */
    private void loadCSVButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadCSVButtonActionPerformed

        //Create a file chooser component
        JFileChooser fc = new JFileChooser();

        //Set the file chooser to only show CSV files by default
        fc.setFileFilter(new CSVFileFilter());

        //Set the title for the file chooser dialog
        fc.setDialogTitle("Choose the CSV file to load users data from");

        //Set the directory of the last loaded file (if there was one)
        if (!filename.equals("")) {

            fc.setCurrentDirectory(new File(filename));
        }

        //Show a file chooser window
        int ret = fc.showOpenDialog(this);

        //Intercept the event corresponding to the approve button
        if (ret == JFileChooser.APPROVE_OPTION) {

            //Try opening the specified file
            openFile(fc.getSelectedFile().getAbsolutePath());
        }
    }//GEN-LAST:event_loadCSVButtonActionPerformed

    /**
     * React to a value change of the users number text box
     * 
     * @param evt The event that happened
     *
     * This method handles a value change of the users number text field. It
     * checks whether the new value is valid or not. If it is, the new value is
     * read and stored into a local variable of the MainWindow object, otherwise
     * the old value is rewritten in the text box.
     */
    private void usersNumberTextPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_usersNumberTextPropertyChange

        try {

            //Update the value of this text box
            usersNumberText.commitEdit();

            //Save the old value
            int prevUsersNumber = usersNumber;

            //Get the new value
            usersNumber = ((Number)usersNumberText.getValue()).intValue();

            //Is the new value valid?
            if (usersNumber < 1) {

                //Reset to a valid value
                usersNumber = prevUsersNumber;

                //Restore the previous value
                usersNumberText.setValue(new Integer(usersNumber));
            }
            else {

                //Valid value. Is it the same as the old one?
                if (prevUsersNumber != usersNumber) {

                    //Different users number
                    changed = true;
                }
            }
        }

        catch (ParseException ex) {

            //Restore the previous value
            usersNumberText.setValue(new Integer(usersNumber));
        }

        //Update the status of all buttons and controls
        updateWindowStatus();
    }//GEN-LAST:event_usersNumberTextPropertyChange

    /**
     * Handles a value change of the coalition number text field
     *
     * @param evt The event that happened to the coalition number text field
     *
     * This method handles a value change of the coalition number text field.
     * It tries to read the new value and checks whether the new value is valid
     * or not. If it is, the new value os saved into the MainWindow object,
     * otherwise the old value is restored.
     */
    private void coalitionNumberTextPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_coalitionNumberTextPropertyChange

        try {

            //Update the value from the text of this text field
            coalitionNumberText.commitEdit();

            //Save the old value
            int prevCoalitionNumber = coalitionNumber;

            //Get the new value
            coalitionNumber = ((Number)coalitionNumberText.getValue()).intValue();

            //Is the new value valid?
            if (coalitionNumber < 1) {

                //Reset to a valid value
                coalitionNumber = prevCoalitionNumber;

                //Restore the previous value
                coalitionNumberText.setValue(new Integer(coalitionNumber));
            }
        }

        catch (ParseException ex) {

            //Restore the previous value
            coalitionNumberText.setValue(new Integer(coalitionNumber));
        }

        //Update the status of all buttons and controls
        updateWindowStatus();
    }//GEN-LAST:event_coalitionNumberTextPropertyChange

    /**
     * Handles a value change in the coalition value text box.
     *
     * @param evt The event happened to the coalition value text box.
     *
     * This method is called to handle a value change to the coalition value
     * text box. The new value is read, then, if it is valid, it is stored in
     * this MainWindow object, otherwise the old value is restored.
     */
    private void coalitionValueTextPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_coalitionValueTextPropertyChange

        try {

            //Read the new value from the text box
            coalitionValueText.commitEdit();

            //Save the old value
            float prevCoalitionSize = coalitionSize;

            //Get the new value
            coalitionSize = (float)((Number)coalitionValueText.getValue()).doubleValue();

            //Is the new value valid?
            if (coalitionSize <= 0.0f) {

                //Reset to a valid value
                coalitionSize = prevCoalitionSize;

                //Restore the previous value
                coalitionValueText.setValue(new Float(coalitionSize));
            }
        }

        catch (ParseException ex) {

            //Restore the previous value
            coalitionValueText.setValue(new Float(coalitionSize));
        }

        //Update the status of all buttons and controls
        updateWindowStatus();
    }//GEN-LAST:event_coalitionValueTextPropertyChange

    /**
     * Handles the quit choice in the drop down menu.
     *
     * @param evt The event happened to the menu item.
     *
     * This method is called to handle an event to the quit menu item. It hides
     * the main window and then terminates the application.
     */
    private void menuQuitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuQuitActionPerformed

        //Close everything

        //Is the algorithm running?
        if (running) {

            //Sop the algorithm
            stopButton.doClick();
        }

        //Hide the main window
        setVisible(false);

        //Destroy the main window
        dispose();

        //Terminate the application
        System.exit(0);
    }//GEN-LAST:event_menuQuitActionPerformed

    /**
     * Handles a click on the start or stop drop down menu item.
     *
     * @param evt The event that happened to the start or stop menu item.
     *
     * This method handles the start and stop drop down menu item. It is
     * implemented by performing a click to either the start or the stop button.
     */
    private void menuStartStopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStartStopActionPerformed

        //Make sure that the item in the menu is enabled
        if (coalitionNumber >= 1 && coalitionSize > 0.0f && usersNumber >= 1 &&
                !filename.equals("")) {

            //Ready to run or running
            if (running) {

                //Running. Click the stop button.
                stopButton.doClick();
            }
            else {

                //Ready to run. Click the start button
                startButton.doClick();
            }
        }
    }//GEN-LAST:event_menuStartStopActionPerformed

    /**
     * Handles the legend drop down menu item.
     *
     * @param evt The event that happened to this menu item.
     *
     * This method implements the legend item of the drop down menu. It is
     * implemented as a click to the legend button.
     */
    private void menuLegendActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuLegendActionPerformed

        //Only click the button if it is enabled (if the menu item is enabled,
        //then the button should be too, but check anyway)
        if (legendButton.isEnabled()) {

            //Perform a click
            legendButton.doClick();
        }
    }//GEN-LAST:event_menuLegendActionPerformed

    /**
     * Handles the open item in the drop down menu
     *
     * @param evt The event that happened to the open item in the drop down menu
     *
     * This method implements the open item in the drop down menu. It implements
     * such an item by performing a click on the load CSV button.
     */
    private void menuOpenActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuOpenActionPerformed

        //Make sure that the button is enabled before performing the click. The
        //button should always be enabled when the menu item is, but check anyway).
        if (loadCSVButton.isEnabled()) {

            //Perform the click
            loadCSVButton.doClick();
        }
    }//GEN-LAST:event_menuOpenActionPerformed

    /**
     * Reloads the users from the CSV file if needed
     *
     * @param rand Specifies whether the function should load the first valid
     * users from the file or whether it should load randomply chosen users.
     *
     * @return true if the users were loaded succesfully or were already loaded
     * and didn't need to be reloaded, or false if the load operation failed
     *
     * This method makes sure that the users array is updated with the specified
     * number of users loaded from the specified file. If the users were already
     * loaded and neither the file name or the numbe of users changed, then this
     * method simply returns true doing nothing. Otherwise it tries to load the
     * specified number of users from the specified file.
     */
    private boolean loadUsers(boolean rand) {
        
        //Update the number of users, the number of coalitions and the size of
        //each coalition from the GUI text fields
        usersNumberTextPropertyChange(null);
        coalitionNumberTextPropertyChange(null);
        coalitionValueTextPropertyChange(null);

        if (!changed) {

            //No need to reload the users array, as no option regarding these
            //changed since the last load operation
            return true;
        }

        try {

            //Try opening the file
            CSVFileReader f = new CSVFileReader(filename);

            //Now try loading the specified number of users
            if (rand) {

                //Remove any previous user
                users.clear();

                //Must load a random selection of users. Start by reading from
                //the file more users than needed
                ArrayList<PowerUser> usersTemp = f.read(usersNumber * 10);

                //Get a random number generator
                Random r = new Random();

                for (int i = 0; i < usersNumber; ++i) {

                    //Get a random index between 0 (inclusive) and the size of
                    //usersTemp (exclusive)
                    int index = r.nextInt(usersTemp.size());

                    //Copy the specified user in the result
                    users.add(usersTemp.get(index));

                    //Remove the user from the usersTemp data structure and repeat
                    usersTemp.remove(index);
                }
            }
            else {

                //Just load the first valid users found
                users = f.read(usersNumber);
            }

            //Close the file
            f.close();

            //Reset the changed flag
            changed = false;
        }
        
        catch (Exception ex) {

            //An error occurred during the above operations
            users = null;
        }

        if (users == null) {

            //Show an error message
            JOptionPane.showMessageDialog(this,
                    "Error while loading the user data from the specified file");

            //Reset the array of users if an error happened
            users = new ArrayList<PowerUser>();

            //Report failure
            return false;
        }
        else {

            //Show a confirmation message box
            JOptionPane.showMessageDialog(this, users.size() + " users loaded");

            //Report success
            return true;
        }
    }

    /**
     * This method is used to save the final solution found by the
     * algorithm
     *
     * @param solution the final solution to be saved
     *
     * This method saves the solution found by the algorithm into this MainWindow
     * object.
     */
    public void saveSolution(ArrayList<Coalition> solution) {

        finalSolution = solution;
    }

    /**
     * Displays the supplied solution in the main window
     *
     * @param solution The solution to be displayed (as an array of coalitions)
     *
     * This method displays a series of bar graphs on the screen to represent
     * the solution provided.
     */
    public void showSolution(Coalition[] solution) {
        
        //Copy everything into an ArrayList
        ArrayList<Coalition> sol = new ArrayList<Coalition>();
        
        //Element by element copy
        for (int i = 0; i < solution.length; ++i) {
            
            sol.add(solution[i]);
        }
        
        //Call the other version of this method
        showSolution(sol);
    }

    /**
     * Displays the supplied solution in the main window
     *
     * @param solution The solution to be displayed (as an ArrayList of
     * coalitions)
     *
     * This method displays a series of bar graphs on the screen to represent
     * the solution provided.
     */
    public void showSolution(ArrayList<Coalition> solution) {

        //Remove all graphs before starting with the new drawing operatins
        mainPanel.removeAll();

        //Create a new panel for each coalition in the solution
        for (int i = 0; i < solution.size(); ++i) {

            //Get this coalition
            Coalition coal = solution.get(i);

            //Create a panel to contain this coalition
            JPanel graph = new JPanel();

            //Set the panel to a grid layout with a single row and unlimited
            //columns
            graph.setLayout(new GridLayout(1, 0));

            //Fill everything with a background color
            graph.setBackground(graphBackground);

            //Set the panel size
            graph.setPreferredSize(new Dimension(graphWidth, graphHeight));
            graph.setMaximumSize(new Dimension(graphWidth, graphHeight));
            graph.setMinimumSize(new Dimension(graphWidth, graphHeight));

            //Add some space above the panel (only for panels after the first one)
            if (i > 0) {
            
                mainPanel.add(Box.createRigidArea(new Dimension(5, 5)));
            }

            //Add the panel to the scrolled area
            mainPanel.add(graph);

            //Get the maximum power consumption for a coalition (it will be used
            //to scale the height of he graph bars)
            float coalitionMax = coal.getMaxConsumptionAllowed();

            //Skip everything if no users are present
            if (coal.numberOfUsers() > 0) {

                //Get the number of time intervals
                int timeIntervals = coal.getUser(0).timeIntervalNumber();

                //Get the horizontal size of each bar
                float barWidth = (float)graphWidth / (float)timeIntervals;

                //Maximum vertical size of each bar
                float maxHeight = (float)graphHeight;

                //Draw each time interval before proceeding to the next one
                for (int t = 0; t < timeIntervals; ++t) {

                    //Create a new panel that will contain a graph bar
                    JPanel bar = new JPanel();

                    //Same color as for the backgorund
                    bar.setBackground(graphBackground);

                    //Only a single column
                    bar.setLayout(new BoxLayout(bar, BoxLayout.Y_AXIS));

                    //Add the panel to the graph
                    graph.add(bar);

                    //Add the empty space to the top of the bar
                    JPanel emptySpace = new JPanel();

                    //Make the empty space the same color as the background
                    emptySpace.setBackground(graphBackground);

                    //0 minimum size
                    emptySpace.setMinimumSize(new Dimension(0, 0));

                    //Add the empty space
                    bar.add(emptySpace);

                    float barHeight;

                    //Add the bar sections for each user in reverse order (the
                    //section for user 0 will be the bottom one)
                    for (int j = coal.numberOfUsers() - 1; j >= 0; --j) {

                        //Create a panel for this bar section
                        JPanel barSection = new JPanel();

                        //Compute the height of this bar section
                        barHeight = coal.getUser(j).usageAtTime(t) * maxHeight / coalitionMax;

                        //Show the user id as the tooltip text
                        barSection.setToolTipText("User " + coal.getUser(j).getUserId());

                        //Set the color
                        barSection.setBackground(coal.getUser(j).getColor());

                        //Set the size of this bar segment
                        barSection.setPreferredSize(new Dimension((int)barWidth, (int)barHeight));
                        barSection.setMaximumSize(new Dimension((int)barWidth, (int)barHeight));
                        barSection.setMinimumSize(new Dimension((int)barWidth, (int)barHeight));

                        //Add the bar section to the bar
                        bar.add(barSection, BorderLayout.SOUTH);
                    }

                }   //for (int t = 0; t < timeIntervals; ++t)

            }   //c.numberOfUsers() > 0

        }   //for (int i = 0; i < solution.size(); ++i)

        //Redraw
        validate();
        repaint();

        //Try to clean up all orphaned objects
        System.gc();
    }

    /**
     * Updates the main window to show no solution.
     *
     * This method is used to modify the main window to show that no solution
     * could be found.
     */
    public final void showNoSolution() {

        //Remove all graphs before starting with the new drawing operatins
        mainPanel.removeAll();

        //Create a label
        JLabel caption = new JLabel();

        //Set the label text
        caption.setText("No solution found");

        //Add the label to the top of the main panel
        mainPanel.add(caption);

        //Redraw
        validate();
        repaint();

        //Cleanup
        System.gc();
    }

    /**
     * Signals that the computation has finished
     *
     * This mehod is used to signal that the computation of the algorithm
     * finished either because a solution (or no solution) was found or because
     * the algorithm was stopped due to au user input.
     */
    public void done() {

        //Set the flag to signal that the algorithm is not running
        running = false;

        //Update the status of all the items on the window
        updateWindowStatus();
    }

    /**
     * Updates the filename used to load users data
     *
     * @param filename The new filename to be used to load users data
     *
     * This method is used to update the filename used to load users data. The
     * window is updated after the new filename has been saved.
     */
    public void openFile(String filename) {

        //Is this a new file?
        if (!filename.equals(this.filename)) {
            
            //The file (and so possibly also the users set) has changed
            changed = true;
        }

        //Save the filename
        this.filename = filename;

        //Update the window components
        updateWindowStatus();
    }

    /**
     * Enables and disables GUI components according to the current
     * status of the application.
     *
     * This method handles all the enable and disable operations that make sure
     * that each button is enabled only when it makes sense for it to be enabled.
     * It also handles the items in the drop down menu and the progress bar.
     */
    private synchronized void updateWindowStatus() {

        //Check the values of the users number, the coalition number, the
        //coalition size and the filename
        if (coalitionNumber >= 1 && coalitionSize > 0.0f && usersNumber >= 1 &&
                !filename.equals("")) {

            //Ready to run or running
            startButton.setEnabled(!running);
            stopButton.setEnabled(running);
            loadCSVButton.setEnabled(!running);
            legendButton.setEnabled(true);
            progressBar.setEnabled(running);
            menuStartStop.setEnabled(true);
            menuLegend.setEnabled(true);
            menuOpen.setEnabled(!running);
        }
        else {

            //Not ready to run
            startButton.setEnabled(false);
            stopButton.setEnabled(false);
            loadCSVButton.setEnabled(true);
            progressBar.setEnabled(false);
            menuStartStop.setEnabled(false);
            menuOpen.setEnabled(true);

            //Can a set of users be loaded?
            if (!filename.equals("") && usersNumber >= 1) {

                //A set of users may be loaded. The legend button and its menu
                //item can be enabled
                legendButton.setEnabled(true);
                menuLegend.setEnabled(true);
            }
            else {

                //No users loadable. Disable the legend button and its menu item
                legendButton.setEnabled(false);
                menuLegend.setEnabled(false);
            }
        }

        //If not running set the progress bar to zero
        if (!running) {

            progressBar.setValue(0);
            progressBar.setIndeterminate(false);
            menuStartStop.setText("Start");
        }
        else {

            menuStartStop.setText("Stop");
        }

        //Redraw the window
        repaint();
    }

    /**
     * Updates the value shown by the progress bar
     *
     * @param progress The progress value to show. Must be between 0 and 100 included
     *
     * This method updates the status of the progress bar. When the bar shows a
     * value of 0, it is set in an indeterminate state o signal that the
     * algorithm is actually running. If the value is more than 0, the value is
     * displayed by the bar.
     */
    public void updateProgressBar(int progress) {

        //Progress value must be between 0 and 100 included
        
        //Keep displaying an indeterminate progress bar if the progress is 0
        if (progress > 0) {

            progressBar.setIndeterminate(false);
            progressBar.setValue(progress);
        }
    }

    /**
     * Entry point of the application
     *
     * @param args Command line arguments. Not used
     *
     * This is the entry point of the application. This procedure simply creates
     * a MainWindow object
     */
    public static void main(String args[]) {

        MainWindow mw = new MainWindow();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JFormattedTextField coalitionNumberText;
    private javax.swing.JFormattedTextField coalitionValueText;
    private javax.swing.JPanel coalitionsInputPanel;
    private javax.swing.JPanel dataInputPanel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JButton legendButton;
    private javax.swing.JPanel legendPanel;
    private javax.swing.JButton loadCSVButton;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem menuLegend;
    private javax.swing.JMenuItem menuOpen;
    private javax.swing.JMenuItem menuQuit;
    private javax.swing.JMenuItem menuStartStop;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JPanel progressBarPanel;
    private javax.swing.JScrollPane scrollPanel;
    private javax.swing.JButton startButton;
    private javax.swing.JButton stopButton;
    private javax.swing.JFormattedTextField usersNumberText;
    // End of variables declaration//GEN-END:variables
}
